@@ -15,6 +15,7 @@ import numpy as np
 from uuid import UUID
 import cv2
 import base64
+from dateutil import parser
 
 
 class NumpyEncoder(json.JSONEncoder):
@@ -246,6 +247,9 @@ class InsertComponent(Component):
             for sub_m in (m.get("map", []) + [m])
         }
         self.input_lookup = {v: k for k, v in self.destination_lookup.items()}
+        self.datatype_lookup = {
+            k: v[1] for t in self.tables for k, v in t.heading.attributes.items()
+        }
 
         if "presets" in self.component_config:
             lcls = locals()
@@ -351,12 +355,38 @@ class InsertComponent(Component):
         # If you have a name mapping it will be applied to each preset
         # Route will 404 if no preset query is defined and 500 if there is an Exception
 
+        # Helper function to convert datetime strings
+        def convert_datetime_string(datetime_string):
+            try:
+                dt = parser.parse(datetime_string)
+                return dt.strftime("%Y-%m-%d %H:%M:%S")
+            except ValueError:
+                raise ValueError("Invalid datetime string")
+
         # Helper function to filter out fields not in the insert,
         # as well as apply the fields_map
-        def filterPreset(preset: dict):
-            return {
-                (self.input_lookup[k] if k in self.input_lookup else k): v
+        def filter_preset(preset: dict):
+            # Any key that follows the schema.table.attribute format,
+            # and its schema.table is not in the forms is filtered out.
+
+            preset_with_tables_filtered = {
+                k: v
                 for k, v in preset.items()
+                if (
+                    len(k.split(".")) == 1
+                    or ".".join(k.split(".")[0:2]) in self.component_config["tables"]
+                )
+            }
+            return {
+                (
+                    self.input_lookup[a]
+                    if (a := k.split(".").pop()) in self.input_lookup
+                    else a
+                ): convert_datetime_string(v)
+                if a in self.datatype_lookup
+                and re.search(r"^date.*|time.*$", self.datatype_lookup[a])
+                else v
+                for k, v in preset_with_tables_filtered.items()
             }
 
         if "presets" not in self.component_config:
@@ -366,9 +396,16 @@ class InsertComponent(Component):
                 {"Content-Type": "text/plain"},
             )
 
-        filtered_preset_dictionary = {
-            k: filterPreset(v) for k, v in self.presets_dict.items()
-        }
+        try:
+            filtered_preset_dictionary = {
+                k: filter_preset(v) for k, v in self.presets_dict.items()
+            }
+        except ValueError as e:
+            return (
+                str(e),
+                406,
+                {"Content-Type": "text/plain"},
+            )
 
         return (
             NumpyEncoder.dumps(filtered_preset_dictionary),
@@ -379,6 +416,7 @@ class InsertComponent(Component):
 
 class TableComponent(FetchComponent):
     attributes_route_format = "{route}/attributes"
+    uniques_route_format = "{route}/uniques"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -449,7 +487,7 @@ class TableComponent(FetchComponent):
 
     def attributes_route(self):
         attributes_meta = _DJConnector._get_attributes(
-            self.fetch_metadata["query"] & self.restriction, include_unique_values=True
+            self.fetch_metadata["query"] & self.restriction
         )
         return (
             NumpyEncoder.dumps(
@@ -462,6 +500,43 @@ class TableComponent(FetchComponent):
             {"Content-Type": "application/json"},
         )
 
+    def uniques_route(self):
+        query = self.fetch_metadata["query"] & self.restriction
+        query_attributes = dict(primary=[], secondary=[])
+        for attribute_name, attribute_info in query.heading.attributes.items():
+            if attribute_info.in_key:
+                query_attributes["primary"].append(
+                    (
+                        [
+                            dict({"text": str(v), "value": v})
+                            for (v,) in (dj.U(attribute_name) & query).fetch()
+                        ]
+                        if True
+                        else None,
+                    )
+                )
+            else:
+                query_attributes["secondary"].append(
+                    (
+                        [
+                            dict({"text": str(v), "value": v})
+                            for (v,) in (dj.U(attribute_name) & query).fetch()
+                        ]
+                        if True
+                        else None,
+                    )
+                )
+
+        return (
+            NumpyEncoder.dumps(
+                dict(
+                    unique_values=query_attributes,
+                )
+            ),
+            200,
+            {"Content-Type": "application/json"},
+        )
+
 
 class MetadataComponent(TableComponent):
     def __init__(self, *args, **kwargs):
